
const extractKeyword = () => {
  const ps = new URLSearchParams(window.location.search)
  const searchType = ps.get('tbm')
  if (searchType) {
    return ''
  }
  return ps.get('q')
}

const testHitsDisplayed = () => {
  return document.body.classList.contains('favalot-watermark')
}

const displayHits = (hits) => {
  document.body.classList.add('favalot-watermark')

  if (! hits.length) return

  const block = document.createElement("div")
  block.classList.add("favalot")
  const title = document.createElement('h3')
  const icon = document.createElement('img')
  icon.src = chrome.runtime.getURL("icon/icon3_on.png")
  title.appendChild(icon)
  const titleText = document.createTextNode('favalotのヒット')
  title.appendChild(titleText)
  block.appendChild(title)
  const ul = document.createElement("ul")
  block.appendChild(ul)
  hits.forEach(hit => {
    const a = document.createElement("a")
    a.setAttribute("href", hit.url)
    const divu = document.createElement('div')
    divu.classList.add("favalot-url")
    divu.textContent = hit.url
    a.appendChild(divu)
    const divt = document.createElement('div')
    divt.classList.add("favalot-text")
    divt.textContent = hit.text
    a.appendChild(divt)
    const li = document.createElement("li")
    li.appendChild(a)
    ul.appendChild(li)
  })

  const rhs = document.getElementById('rhs')
  if (rhs) {
    rhs.prepend(block)
  } else {
    const myRhs = document.createElement("div")
    myRhs.classList.add('favalot-rhs')
    myRhs.appendChild(block)
    const rcnt = document.getElementById('rcnt')
    rcnt.appendChild(myRhs)
  }
}

const run = async () => {
  if (testHitsDisplayed()) {
    // do nothing
    return
  }
  
  const keyword = extractKeyword()
  if (! keyword) {
    return
  }
  const message = {type:'search', keyword}
  const {ok, value} = await chrome.runtime.sendMessage(message)
                      .catch((e) => {
                        // ignore error; sometime we got "Error: Could not establish connection. Receiving end does not exist."
                        return {ok:false, value:null}
                      })
  if (ok) {
    displayHits(value)
  }
}

run()