

//const imagesOn = {'128': '../icon/icon_on_128.png'}
//const imagesOff = {'128': '../icon/icon_off_128.png'}
const imagesOn = {'128': '../icon/icon3_on.png'}
const imagesOff = {'128': '../icon/icon3_off.png'}
const imagesMuted = {'128': '../icon/icon3_muted.png'}

const setIcon = async (faved, tabId) => {
  const image = faved ? imagesOn : imagesOff
  await chrome.action.setIcon({path:image, tabId})
}

const testUrl = (url) => {
  return (url.startsWith('http://') || url.startsWith('https://'))
}

const init = (dispatch) => {
  chrome.action.onClicked.addListener(async (tab) => {
    const message = {type:'favToggle', url:tab.url}
    const {ok, value} = await dispatch(message)
    if (ok) {
      const faved = value
      await setIcon(faved, tab.id)
    }
  })

  const sync = async (tabId, url) => {
    if (testUrl(url)) {
      await chrome.action.enable(tabId)
      const message = {type:'favTest', url}
      const {ok, value} = await dispatch(message)
      if (ok) {
        const faved = value
        await setIcon(faved, tabId)
      }
    } else {
      await chrome.action.disable(tabId)
      await chrome.action.setIcon({path:imagesMuted, tabId})
    }
  }
  
  chrome.tabs.onUpdated.addListener(async (tabId, changeInfo, tab) => {
    if ('url' in changeInfo || 'status' in changeInfo) {
      await sync(tabId, tab.url)
      .catch((e) => {
        // ignore error; sometime We got "Error: No tab with id: XXX".
      })
    }
  })

  chrome.tabs.onActivated.addListener(async (activeInfo) => {
    const tab = await chrome.tabs.get(activeInfo.tabId)
    await sync(tab.id, tab.url)
  })

  setTimeout(async () => {
    const tabs = await chrome.tabs.query({active:true, currentWindow:true})
    for (let tab of tabs) {
      await sync(tab.id, tab.url)
    }
  }, 0)
}

export const all = {init}