

const MAX_BUFLEN = 100
const buffer = new Array(MAX_BUFLEN)
let offset = 0
let sendToConsole = false

const init = (aSendToConsole) => {
  buffer.fill(null)
  sendToConsole = aSendToConsole
}

const add = (level, message, extra) => {
  buffer[offset] = {when:Date.now(), level, message, extra}
  offset = (offset + 1) % MAX_BUFLEN
  if (sendToConsole || level == 'error') {
    console[level.toLowerCase()](message, extra)
  }
}

const error = (message, extra) => {
  add('ERROR', message, extra)
}

const warning = (message, extra) => {
  add('WARNING', message, extra)
}

const info = (message, extra) => {
  add('INFO', message, extra)
}

const log = (message, extra) => {
  add('LOG', message, extra)
}

const flattenError = (err) => {
  if (err instanceof Error) {
    return {name:err.name, message:err.message, stack:err.stack}
  } else {
    return err
  }
}

const dump = () => {
  const rv = []
  for (let i = 0; i < MAX_BUFLEN; i++) {
    const j = (i + offset + 1) % MAX_BUFLEN
    if (! buffer[j]) continue
    rv.push(buffer[j])
  }
  return rv
}

export const all = {init, error, warning, info, log, flattenError, dump}
